/**
 * @function crawlNichan
 */
'use strict'

const co = require('co')
const czombie = require('czombie')
const asleep = require('asleep')
const { parse: parseUrl } = require('url')
const { CrawlTypeCodes } = require('ore-fol-codes')
const { v4: generateUUID } = require('uuid')
const sleepSome = () => asleep(500 * Math.random() + 100)

const skipError = (err) => {
  console.warn(`[crawlNichan] Skip Error: "${err.message}`)
}

/** @lends crawlNichan */
function crawlNichan (url) {
  return co(function * () {
    const { host, protocol } = parseUrl(url)
    let isOpen2ch = /open2ch\.net$/.test(host)
    if (!isOpen2ch) {
      throw new Error('Invalid url')
    }
    let browser = yield czombie()
    yield browser.visit(url).catch(skipError)
    yield sleepSome()
    let { document, body } = browser
    let { title } = document
    let favicon = `${protocol}//${host}/favicon.ico`
    let result = {
      url,
      favicon,
      title,
      thread: {
        name: title,
        uuid: generateUUID(),
        description: null,
        createdAt: new Date(),
        additionalInfo: {
          crawled: true,
          source: {
            url,
            favicon,
            title
          }
        }
      },
      posts: []
    }
    switch (host) {
      case 'http://hayabusa.open2ch.net/':
      default: {
        let threadTitle = body.querySelector('.threadTitle')
        result.thread.description = threadTitle && threadTitle.textContent || null
        let dls = body.querySelector('.thread').querySelectorAll('dl')
        for (let dl of dls) {
          let dt = dl.querySelector('dt')
          let dd = dl.querySelector('dd')
          let number = dt.getAttribute('res')
          let imgs = dd.querySelectorAll('img')
          if (imgs) {
            for (let img of imgs) {
              let textNode = dd.childNodes[ 0 ]
              result.posts.push({
                uuid: generateUUID(),
                type: CrawlTypeCodes.CRAWL_POST,
                text: String(textNode && textNode.textContent || '').trim() || null,
                image: img.src,
                additionalInfo: {
                  crawled: true,
                  source: {
                    url,
                    name: `>>${number} - ${title}`
                  }
                }
              })
            }
          }
        }
        let success = result.thread && (result.posts.length > 0)
        if (!success) {
          throw new Error('Failed to crawl nichan')
        }
      }
    }
    return result
  })
}

module.exports = crawlNichan
